# -*- coding: utf-8 -*-
from trytond.pool import Pool
from stock import ShipmentOut, Move
from report_html_stock import PickingList, SupplierRestockingList, \
    CustomerReturnRestockingList, ConsolidatedPickingList, DeliveryNote


def register():
    Pool.register(
        ShipmentOut,
        Move,
        module='report_html_stock', type_='model'
    )
    Pool.register(
        PickingList,
        SupplierRestockingList,
        CustomerReturnRestockingList,
        ConsolidatedPickingList,
        DeliveryNote,
        module='report_html_stock', type_='report'
    )



# -*- coding: utf-8 -*-
import sys
import os
import unittest
from decimal import Decimal

import trytond.tests.test_tryton
from trytond.tests.test_tryton import POOL, USER, CONTEXT
from trytond.transaction import Transaction
from trytond.pyson import Eval

DIR = os.path.abspath(os.path.normpath(os.path.join(
    __file__, '..', '..', '..', '..', '..', 'trytond'
)))
if os.path.isdir(DIR):
    sys.path.insert(0, os.path.dirname(DIR))

ROOT_JSON_FOLDER = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), 'json_data'
)


class BaseTestCase(unittest.TestCase):
    '''
    Base Test Case for report_html_stock module.
    '''

    def setUp(self):
        """
        Set up data used in the tests.
        this method is called before each test function execution.
        """
        trytond.tests.test_tryton.install_module('report_html_stock')

        self.Currency = POOL.get('currency.currency')
        self.Company = POOL.get('company.company')
        self.Party = POOL.get('party.party')
        self.User = POOL.get('res.user')
        self.ProductTemplate = POOL.get('product.template')
        self.Uom = POOL.get('product.uom')
        self.ProductCategory = POOL.get('product.category')
        self.Product = POOL.get('product.product')
        self.Country = POOL.get('country.country')
        self.Subdivision = POOL.get('country.subdivision')
        self.Employee = POOL.get('company.employee')
        self.Location = POOL.get('stock.location')
        self.ShipmentOut = POOL.get('stock.shipment.out')
        self.StockLocation = POOL.get('stock.location')
        self.Address = POOL.get('party.address')
        self.Move = POOL.get('stock.move')
        self.Product = POOL.get('product.product')
        self.ShipmentIn = POOL.get('stock.shipment.in')
        self.ShipmentOutReturn = POOL.get('stock.shipment.out.return')

    def _create_coa_minimal(self, company):
        """Create a minimal chart of accounts
        """
        AccountTemplate = POOL.get('account.account.template')
        Account = POOL.get('account.account')

        account_create_chart = POOL.get(
            'account.create_chart', type="wizard")

        _, account_template = AccountTemplate.search(
            [('parent', '=', None)]
        )

        session_id, _, _ = account_create_chart.create()
        create_chart = account_create_chart(session_id)
        create_chart.account.account_template = account_template
        create_chart.account.company = company
        create_chart.transition_create_account()

        receivable, = Account.search([
            ('kind', '=', 'receivable'),
            ('company', '=', company),
        ])
        payable, = Account.search([
            ('kind', '=', 'payable'),
            ('company', '=', company),
        ])
        create_chart.properties.company = company
        create_chart.properties.account_receivable = receivable
        create_chart.properties.account_payable = payable
        create_chart.transition_create_properties()

    def _get_account_by_kind(self, kind, company=None, silent=True):
        """Returns an account with given spec
        :param kind: receivable/payable/expense/revenue
        :param silent: dont raise error if account is not found
        """
        Account = POOL.get('account.account')
        Company = POOL.get('company.company')

        if company is None:
            company, = Company.search([], limit=1)

        accounts = Account.search([
            ('kind', '=', kind),
            ('company', '=', company)
        ], limit=1)
        if not accounts and not silent:
            raise Exception("Account not found")

        if not accounts:
            return None
        account, = accounts

        return account

    def _create_payment_term(self):
        """Create a simple payment term with all advance
        """
        PaymentTerm = POOL.get('account.invoice.payment_term')

        return PaymentTerm.create([{
            'name': 'Direct',
            'lines': [('create', [{'type': 'remainder'}])]
        }])

    def setup_defaults(self):
        """Creates default data for testing
        """

        self.currency, = self.Currency.create([{
            'name': 'US Dollar',
            'code': 'USD',
            'symbol': '$',
        }])

        with Transaction().set_context(company=None):
            self.company_party, = self.Party.create([{
                'name': 'openlabs',
            }])

        self.company, = self.Company.create([{
            'party': self.company_party,
            'currency': self.currency,
        }])
        self.User.write([self.User(USER)], {
            'company': self.company,
            'main_company': self.company,
        })

        CONTEXT.update(self.User.get_preferences(context_only=True))

        self.country, = self.Country.create([{
            'name': 'United States of America',
            'code': 'US',
        }])
        self.subdivision, = self.Subdivision.create([{
            'country': self.country.id,
            'name': 'California',
            'code': 'CA',
            'type': 'state',
        }])

        self.party, = self.Party.create([{
            'name': 'Bruce Wayne',
            'addresses': [('create', [{
                'name': 'Bruce Wayne',
                'party': Eval('id'),
                'city': 'Gotham',
                'country': self.country.id,
                'subdivision': self.subdivision.id,
            }])],
            'contact_mechanisms': [('create', [
                {'type': 'mobile', 'value': '8888888888'},
            ])],
        }])
        self.Party.write([self.company.party], {
            'addresses': [('create', [{
                'name': 'Lie Nielsen',
                'city': 'Los Angeles',
                'country': self.country.id,
                'subdivision': self.subdivision.id,
            }])],
        })
        self.product_category, = self.ProductCategory.create([{
            'name': 'Automobile',
        }])
        self.uom, = self.Uom.search([('name', '=', 'Unit')])

        self.product_template, = self.ProductTemplate.create([{
            'name': 'Bat Mobile',
            'type': 'goods',
            'category': self.product_category.id,
            'list_price': Decimal('20000'),
            'cost_price': Decimal('15000'),
            'default_uom': self.uom.id,
        }])
        self.product, = self.Product.create([{
            'template': self.product_template.id,
            'code': '123',
        }])



# -*- coding: utf-8 -*-
from itertools import groupby, imap, chain

from trytond.pool import Pool, PoolMeta
from trytond.transaction import Transaction

from openlabs_report_webkit import ReportWebkit

__all__ = [
    'PickingList', 'SupplierRestockingList', 'CustomerReturnRestockingList',
    'ConsolidatedPickingList'
]
__metaclass__ = PoolMeta


class ReportMixin(ReportWebkit):
    """
    Mixin Class to inherit from, for all HTML reports.
    """

    @classmethod
    def wkhtml_to_pdf(cls, data, options=None):
        """
        Call wkhtmltopdf to convert the html to pdf
        """
        Company = Pool().get('company.company')

        company = ''
        if Transaction().context.get('company'):
            company = Company(Transaction().context.get('company')).party.name
        options = {
            'margin-bottom': '0.50in',
            'margin-left': '0.50in',
            'margin-right': '0.50in',
            'margin-top': '0.50in',
            'footer-font-size': '8',
            'footer-left': company,
            'footer-line': '',
            'footer-right': '[page]/[toPage]',
            'footer-spacing': '5',
        }
        return super(ReportMixin, cls).wkhtml_to_pdf(
            data, options=options
        )

    @classmethod
    def get_sorted_moves(cls, records):
        """
        Sorting the moves for each shipment
        """
        sorted_moves = {}
        for shipment in records:
            sorted_moves[shipment.id] = sorted(
                shipment.inventory_moves,
                key=lambda m: (m.from_location, m.to_location)
            )
        return sorted_moves


class PickingList(ReportMixin):
    """
    Picking List Report
    """
    __name__ = 'report.picking_list'

    @staticmethod
    def sort_inventory_moves(shipment, sort_key=None):
        """
        A sorter that can be overwritten by downstream modules
        """
        return sorted(shipment.inventory_moves, key=sort_key)

    @classmethod
    def parse(cls, report, records, data, localcontext):
        sort_fn = lambda shipment, sort_key=None: sorted(
            shipment.inventory_moves, key=sort_key
        )
        localcontext['sort_inventory_moves'] = sort_fn
        localcontext['sort_key'] = lambda move: (
            move.from_location.rec_name, move.product.name
        )
        return super(PickingList, cls).parse(
            report, records, data, localcontext
        )


class ConsolidatedPickingList(ReportMixin):
    """
    Consolidated Picking List.
    """
    __name__ = 'report.consolidated_picking_list'

    @classmethod
    def group_key(cls, move):
        """
        Key function for grouping and sorting of
        moves
        """
        return (move.from_location, move.product)

    @classmethod
    def get_product_repr_from(cls, key):
        """
        Returns the product representation from the key
        """
        return key[1].rec_name

    @classmethod
    def get_location_repr_from(cls, key):
        """
        Returns the location representation from the key
        """
        return key[0].rec_name

    @classmethod
    def parse(cls, report, records, data, localcontext):
        """
        The default implementation groups by product
        and sorts by from_location.
        """
        localcontext['grouped_moves'] = []
        for key, grouper in groupby(
                # Sort all the moves from all shipments
                # and then group it
                list(sorted(
                    # Chain all inventory moves from all shipments
                    chain(*imap(lambda s: s.inventory_moves, records)),
                    key=cls.group_key
                )), cls.group_key):
            moves = list(grouper)
            # TODO: Sum totals everything, the UOM to base UOM conversion
            # is not done
            localcontext['grouped_moves'].append(
                (key, moves, sum(map(lambda m: m.quantity, moves)))
            )

        localcontext['get_product_repr_from'] = cls.get_product_repr_from
        localcontext['get_location_repr_from'] = cls.get_location_repr_from
        return super(ConsolidatedPickingList, cls).parse(
            report, records, data, localcontext
        )


class SupplierRestockingList(ReportMixin):
    'Supplier Restocking List'
    __name__ = 'report.supplier_restocking_list'

    @classmethod
    def parse(cls, report, records, data, localcontext):

        sorted_moves = cls.get_sorted_moves(records)

        localcontext['moves'] = sorted_moves

        return super(SupplierRestockingList, cls).parse(
            report, records, data, localcontext
        )


class CustomerReturnRestockingList(ReportMixin):
    'Customer Return Restocking List'
    __name__ = 'report.customer_return_restocking_list'

    @classmethod
    def parse(cls, report, records, data, localcontext):

        sorted_moves = cls.get_sorted_moves(records)

        localcontext['moves'] = sorted_moves

        return super(CustomerReturnRestockingList, cls).parse(
            report, records, data, localcontext
        )


class DeliveryNote(ReportMixin):
    "Delivery Note"
    __name__ = 'report.delivery_note'



# -*- coding: utf-8 -*-
"""
    :copyright: (c) 2015 by Fulfil.IO Inc.
    :license: BSD, see LICENSE for more details.
"""
from trytond.model import fields
from trytond.pool import PoolMeta, Pool

__metaclass__ = PoolMeta
__all__ = ['ShipmentOut', 'Move']


class ShipmentOut:
    __name__ = 'stock.shipment.out'

    sales = fields.Function(
        fields.One2Many('sale.sale', None, 'Sales'),
        'get_sales'
    )

    def get_sales(self, name=None):
        """
        Returns sales associated with a shipment.
        """
        sales = set([m.sale.id for m in self.moves if m.sale])
        return list(sales)

    def _get_inventory_move(self, move):
        """
        Write origin also to inventory moves
        """
        new_move = super(ShipmentOut, self)._get_inventory_move(move)

        # Added a field inventory_origin for the reports to keep track of
        # corresponding outgoing move's origin
        # This is particularly used in picking list report where we need
        # to print inventory moves and we need corresponding outgoing
        # moves origin
        new_move.inventory_origin = move.origin
        return new_move


class Move:
    __name__ = 'stock.move'

    inventory_origin = fields.Reference(
        'Inventory Origin', selection='get_origin', readonly=True
    )
    sale_order = fields.Function(
        fields.Many2One('sale.sale', 'Sale'), 'get_sale_order'
    )

    def get_sale_order(self, name):
        """
        Get sale order from move origin
        """
        SaleLine = Pool().get('sale.line')
        ShipmentOut = Pool().get('stock.shipment.out')

        if not self.shipment:
            return None

        origin = self.origin or self.inventory_origin

        if isinstance(self.shipment, ShipmentOut) and \
                isinstance(origin, SaleLine):
            return origin.sale.id



# -*- coding: utf-8 -*-
import sys
import os
import unittest

import trytond.tests.test_tryton
from trytond.tests.test_tryton import POOL, DB_NAME, USER, CONTEXT
from trytond.transaction import Transaction
from trytond.pool import Pool

from test_base import BaseTestCase

DIR = os.path.abspath(os.path.normpath(os.path.join(
    __file__, '..', '..', '..', '..', '..', 'trytond'
)))
if os.path.isdir(DIR):
    sys.path.insert(0, os.path.dirname(DIR))


class TestShipment(BaseTestCase):
    """
    Test Picking List
    """

    @unittest.skipIf(sys.platform == 'darwin', 'wkhtmltopdf repo on OSX')
    def test_0110_test_picking_list_report(self):
        """
        Test picking list report
        """
        Report = POOL.get('report.picking_list', type="report")
        Date = POOL.get('ir.date')
        ActionReport = POOL.get('ir.action.report')

        with Transaction().start(DB_NAME, USER, context=CONTEXT):
            self.setup_defaults()

            with Transaction().set_context({'company': self.company.id}):
                shipment, = self.ShipmentOut.create([{
                    'planned_date': Date.today(),
                    'effective_date': Date.today(),
                    'customer': self.party.id,
                    'warehouse': self.StockLocation.search([
                        ('type', '=', 'warehouse')
                    ])[0],
                    'delivery_address': self.party.addresses[0],
                }])
                move1, = self.Move.create([{
                    'shipment': ('stock.shipment.out', shipment.id),
                    'product': self.product.id,
                    'uom': self.uom.id,
                    'quantity': 6,
                    'from_location': shipment.warehouse.storage_location.id,
                    'to_location': shipment.warehouse.output_location.id,
                }])

                # Change the report extension to PDF
                action_report, = ActionReport.search([
                    ('name', '=', 'Picking List'),
                    ('report_name', '=', 'report.picking_list')
                ])
                action_report.extension = 'pdf'
                action_report.save()

                # Set Pool.test as False as we need the report to be generated
                # as PDF
                # This is specifically to cover the PDF coversion code
                Pool.test = False

                # Generate Picking List Report
                val = Report.execute([shipment.id], {})

                # Revert Pool.test back to True for other tests to run normally
                Pool.test = True

                self.assertTrue(val)
                # Assert report type
                self.assertEqual(val[0], 'pdf')
                # Assert report name
                self.assertEqual(val[3], 'Picking List')

    @unittest.skipIf(sys.platform == 'darwin', 'wkhtmltopdf repo on OSX')
    def test_0120_test_supplier_restocking_list_report(self):
        """
        Test Supplier Restocking list report
        """
        Report = POOL.get('report.supplier_restocking_list', type="report")
        Date = POOL.get('ir.date')
        ActionReport = POOL.get('ir.action.report')

        with Transaction().start(DB_NAME, USER, context=CONTEXT):
            self.setup_defaults()

            with Transaction().set_context({'company': self.company.id}):
                shipment, = self.ShipmentIn.create([{
                    'planned_date': Date.today(),
                    'effective_date': Date.today(),
                    'supplier': self.party.id,
                    'warehouse': self.StockLocation.search([
                        ('type', '=', 'warehouse')
                    ])[0],
                }])
                move1, = self.Move.create([{
                    'shipment': ('stock.shipment.in', shipment.id),
                    'product': self.product.id,
                    'uom': self.uom.id,
                    'quantity': 6,
                    'from_location': shipment.warehouse.input_location.id,
                    'to_location': shipment.warehouse.storage_location.id,
                }])

                # Change the report extension to PDF
                action_report, = ActionReport.search([
                    ('name', '=', 'Supplier Restocking List'),
                    ('report_name', '=', 'report.supplier_restocking_list')
                ])
                action_report.extension = 'pdf'
                action_report.save()

                # Set Pool.test as False as we need the report to be generated
                # as PDF
                # This is specifically to cover the PDF coversion code
                Pool.test = False

                # Generate Supplier Restocking List Report
                val = Report.execute([shipment.id], {})

                # Revert Pool.test back to True for other tests to run normally
                Pool.test = True

                self.assertTrue(val)
                # Assert report type
                self.assertEqual(val[0], 'pdf')
                # Assert report name
                self.assertEqual(val[3], 'Supplier Restocking List')

    @unittest.skipIf(sys.platform == 'darwin', 'wkhtmltopdf repo on OSX')
    def test_0130_test_Customer_restocking_list_report(self):
        """
        Test Supplier Restocking list report
        """
        Report = POOL.get('report.customer_return_restocking_list',
                          type="report")
        Date = POOL.get('ir.date')
        ActionReport = POOL.get('ir.action.report')

        with Transaction().start(DB_NAME, USER, context=CONTEXT):
            self.setup_defaults()

            with Transaction().set_context({'company': self.company.id}):
                shipment, = self.ShipmentOutReturn.create([{
                    'planned_date': Date.today(),
                    'effective_date': Date.today(),
                    'customer': self.party.id,
                    'warehouse': self.StockLocation.search([
                        ('type', '=', 'warehouse')
                    ])[0],
                    'delivery_address': self.party.addresses[0],
                }])
                move1, = self.Move.create([{
                    'shipment': ('stock.shipment.out.return', shipment.id),
                    'product': self.product.id,
                    'uom': self.uom.id,
                    'quantity': 6,
                    'from_location': shipment.warehouse.input_location.id,
                    'to_location': shipment.warehouse.storage_location.id,
                }])

                # Change the report extension to PDF
                action_report, = ActionReport.search([
                    ('name', '=', 'Customer Restocking List'),
                    ('report_name', '=',
                     'report.customer_return_restocking_list')
                ])
                action_report.extension = 'pdf'
                action_report.save()

                # Set Pool.test as False as we need the report to be generated
                # as PDF
                # This is specifically to cover the PDF coversion code
                Pool.test = False

                # Generate Customer Restocking List Report
                val = Report.execute([shipment.id], {})

                # Revert Pool.test back to True for other tests to run normally
                Pool.test = True

                self.assertTrue(val)
                # Assert report type
                self.assertEqual(val[0], 'pdf')
                # Assert report name
                self.assertEqual(val[3], 'Customer Restocking List')


def suite():
    "Define suite"
    test_suite = trytond.tests.test_tryton.suite()
    test_suite.addTests(
        unittest.TestLoader().loadTestsFromTestCase(TestShipment)
    )
    return test_suite


if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())
